//****************************************************************************
//**
//**    VecOCS.cpp
//**    Source - Simple example of using the mVec3 and mOCS classes.
//**    Copyright (C)2000 Douglas H. Cox
//**
//**    DHC - 8/3/2000
//**
//****************************************************************************
//============================================================================
//    Headers
//============================================================================
#include <mMath.h>
#include <mVec3.h>
#include <mOCS.h>
#include <mMatrix.h>

#include <stdio.h>
//============================================================================
//    Private Defines / Enums / Typedefs
//============================================================================

//============================================================================
//    Private Structs / Functions / Utility Classes
//============================================================================

//============================================================================
//    Private Data
//============================================================================

//============================================================================
//    Function Implementations 
//============================================================================

//=============================================================
// vec3Test
void vec3Test()
{
  // mVec3 functions
  mVec3 a, b, r;

  a.set(1.5f, 0.0f, 0.0f);
  a.normalize();

  b.set(0.0f, 2.75f, 0.0f);
  b.normalize();

  r.cross(a, b); // should be pointing in +z direction
  
}

//=============================================================
// ocsTest
void ocsTest()
{
  mOCS parentOCS(mVec3(10.0f, 1.0f, -10.0f), mFrame(), mVec3(), mOCS::ORIGIN);
  mOCS childOCS(mVec3(20.0f, 5.0f, -30.0f),  mFrame(), mVec3(), mOCS::ORIGIN);
  // ^ construct 2 ocs's with only the origin component not an identity value.
  mOCS r;

  unsigned parentType = parentOCS.getType();
  // ^ should = mOCS::ORIGIN

  r.xformFrom(parentOCS, childOCS);

  parentOCS.setOrigin(0.0f, 0.0f, 0.0f);
  parentOCS.setHP(90.0f, 0.0f);
  // ^ Heading=Rotation about Z axis,
  // ^ Pitch=Rotation about the X axis,
  //   Roll=Rotation about the Y axis.
  
  parentType = parentOCS.getType();
  // ^ should = mOCS::ORIGIN | mOCS::FRAME, because the above setHP
  // modifies the frame component.


  r.xformFrom(childOCS, parentOCS);
  // r can now be thought of as the world->child OCS

  mVec3 a(1.0f, 2.0f, 3.0f);
  mVec3 b;
  mVec3 rv1, rv2;

  rv1.xformFrom(a, childOCS);
  // rv1 is now positioned in the parentOCS
  rv1.xformFrom(rv1, parentOCS);
  // rv1 is now positioned in the world OCS

  rv2.xformFrom(a, r);

  // assert(rv1 == rv2);
  // NOTE: ^ may assert because of precision errors.  
  // Viewing the variables in the watch window shows they should be the same.
  assert(rv1.approxEqual(rv2));
  // ^ mVec*.approxEqual and mApproxEqual should be used when comparing
  // transformed points.

  b.xformTo(rv1, r); // go from a point in the world ocs to child ocs.

  // assert(b == a); 
  // NOTE: ^ may assert because of precision errors.  
  // Viewing the variables in the watch window shows they should be the same.
  
  assert(b.approxEqual(a));
  // ^ mVec*.approxEqual and mApproxEqual should be used when comparing
  // transformed points.
  
}

//=============================================================
// main
void main()
{
  mInitialize(); // * Must be called 1st.

  // Simple mVec3 testing.
  vec3Test();

  // Simple mOCS testing.
  ocsTest();


  mUninitialize();
}

//****************************************************************************
//**    End - VecOCS.cpp
//****************************************************************************

